//
//  Animation Example
//  guit GUI Toolkit
//  Copyright © 2019 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paristech.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

class Animations : public GWindow {
public:
  Animations() : GWindow("#animations Aminations") {

    auto& anim = Anim(10'000)
    << (AnimStep() << Pos(10,10)   << Color("blue")   << "You like potato")
    << (AnimStep() << Pos(220,100) << Color("green")  << "And I like potahto")
    << (AnimStep() << Pos(50,150)  << Color("orange") << "You like tomato")
    << (AnimStep() << Pos(5,30)    << Color("pink")   << "And I like tomahto")
    << (AnimStep() << Pos(220,120) << Color("red")    << "Potato, potahto, tomato, tomahto!")
    << (AnimStep() << Pos(50,150)  << Color("purple") << "Let's call the whole thing off!")
    << (AnimStep() << Pos(10,10)   << Color("blue")   << "You like potato");

    auto& anim_area = VBox("{size: 400 200; border:rounded; background:white}")
    << (Label("{font: bold 20 Arial, sans-serif}") << anim);

    auto& anim_ctrl = HBox("{align: c c}")
    << (Button("{size:250 a; font: 16 Arial, sans-serif} Move Text!")
        << Check(false)
        << on::checked   / [&]{anim.start();}
        << on::unchecked / [&]{anim.stop();}
        << on::mouseover / "Click Now!" / (Anim(700) << Color("pink") << Color("red"))
        << on::checked   / "Stop Moving Text!" / (Anim(1500) << Align("start") << Align("end"))
        );

    // - - - - - -

    auto& anim1 = Anim(1000, 1)
    << (AnimStep() << GBackground::red    << GColor::white << "one")
    << (AnimStep() << GBackground::yellow << GColor::blue  << "two")
    << (AnimStep() << GBackground::red    << GColor::white << "three");

    auto& anim2 = Anim(1000)
    << (AnimStep() << GBackground::red    << GColor::white << "one")
    << (AnimStep() << GBackground::yellow << GColor::blue  << "two")
    << (AnimStep() << GBackground::red    << GColor::white << "three");

    auto& anim3 = Anim(1000)
    << (AnimStep() << GBackground::red    << GColor::white << "one")
    << (AnimStep() << GBackground::yellow << GColor::blue  << "two")
    << (AnimStep() << GBackground::red    << GColor::white << "three");


    // alternate technique
    auto& anim4_button = Button("Anim on Click 2");
    anim4_button
    << on::action    / [&]{ anim4_button << GBackground::red << "one"; }
    / CallWait(1000) / [&]{ anim4_button << GBackground::yellow << "two"; }
    / CallWait(1000) / [&]{ anim4_button << GBackground::red << "three"; }
    / CallWait(1000) / [&]{ anim4_button << GBackground::white << "Anim on Click 2"; };


    auto& postpone_btn = Button("Postpone");
    postpone_btn
    << on::mousedown / [&]{ postpone_btn = "<c:red>Starting in 1sec..."; }
    << on::action
    / CallWait(1000) / [&]{ postpone_btn << "<c:blue> Action Now!"; }
    / CallWait(1000) / [&]{ postpone_btn << "<c:red> Action Done!"; }
    / CallWait(1000) / [&]{ postpone_btn << "Postpone"; };


    auto& right_panel = VBox("{align:s x; gap:10; size:a a; flex:0 0}")
    << (Button("  Anim on Enter  ") << on::mouseover / anim3)
    << (Button("  Anim on Press  ") << on::mousedown / anim2)
    << (Button("  Anim on Click  ") << on::action    /anim1)
    << Separator
    << anim4_button
    << postpone_btn;

    *this
    << (VBox
        << anim_area
        << anim_ctrl
        )
    << right_panel;
  }
};


GWindow& Examples::makeAnimations() {
  return *new Animations;
}
