//
//  Guit GUI Toolkit Examples.
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace guit;
using namespace std;

class Charts : public GWindow {
  GFloat selvalue;                // value of selected item
  GBackground selcolor;           // bgcolor of selected item
  GChartBox chart{"#chart Chart"};      // barchart or piechart
  GCheck show_rightpanel{true};   // controls whether right panel is shown

public:
  Charts() : GWindow("#chartwin Charts")
  {
    chart.setRange(0.f, 100.f);

    auto& opendial = OpenDialog("Open File", ":/data");
    opendial << [&]{chart.read(opendial.pathname());};

    auto& savedial = SaveDialog("Save File", "~");
    savedial << [&]{chart.write(savedial.pathname(), GIOMode::Content);};

    auto& toolbar = ToolBar("#toolbar")
    << ("{icon:open.png} Open"   << opendial)
    << ("{icon:save.png} Save"   << savedial)
    << ("{icon:print.png} Print" << [this]{chart.write(std::cout, GIOMode::Content);})
    // << ("{icon:trash.png} Clear" << [this]{chart.removeChildren();})
    << Filler
    << (ChoiceItem()    // this item controls whether the right panel is shown
        << show_rightpanel
        << on::checked   / "<b> > "
        << on::unchecked / "<b> < "
        );

    *this
    << toolbar
    << chart
    << (VBox("#rightpanel")
        << (Show() <<= show_rightpanel)
        << (ChoiceBox(".panel {flow:row} <b>Type")
            << Choice(0)
            << (Button("Vert.")  << [&]{chart << GFlow::row;})
            << (Button("Horiz.") << [&]{chart << GFlow::column;})
            << (Button("Pie")    << [&]{chart << GFlow::pie;})
            )
        << (Table(".panel <b>Value")
            << makeSlider("V", selvalue)
            )
        << (Table(".panel <b>Colors")
            << makeSlider("R", selcolor.redField())
            << makeSlider("G", selcolor.greenField())
            << makeSlider("B", selcolor.blueField())
            << makeSlider("A", selcolor.alphaField())
            )
        );

    bindChart();
  }

  void bindChart() {
    // when a new chart item is selected:
    // 1) unbind previous item
    // 2) bind new item if not null

    chart << on::change / [&]{
      gunbind(selcolor, 1);
      gunbind(selvalue, 1);

      if (auto* item = chart.chosen()) {
        gsync(selcolor, item->prop<GBackground>(), 1);
        gsync(selvalue, item->prop<GFloat>(), 1);      // !!!! GValue !!!!!
      }
    };
  }

  Gadget& makeSlider(GString const& name, GFloat& value) {
    return HBox(".textslider")
   << Label(name)
    << (Slider << value)
    << (TextField << value);
  }

  Gadget& makeSlider(GString const& name, GColor::Field const& field) {
    // colors are in range [0, 1]
    auto& value = *new GFloat(0., 0., 1.);
    // sync value with field
    value ^= field;
    return makeSlider(name, value);
  }
};


GWindow& Examples::makeCharts() {
  return *new Charts();
}
