//
//  Control Panel with State Machine
//  guit GUI Toolkit
//  Copyright © 2019/2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <gstatechart.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;


class ControlPanel : public GWindow {
  // states and state machine
  // each state corresponds to a page that is displayed when the state is entered
  GStateChart sc;
  GState state1{sc}, state2a{sc}, state2b{sc}, state3a{sc}, state3b{sc}, state4{sc};

  GChoice service, light, channel, lang;
  GText light_txt, channel_txt, lang_txt;
  GRegText from{"00:00"}, to{"23:59"};  // selected time range

  GProp& blink = on::enabled / [](GEvent&e){e.from().blink();};

public:  
  ControlPanel() : GWindow("#control-panel Control panel") {
    service = 1;
    light = 0;
    channel = 0;
    lang = 2;
    from.setTimeFormat();   // must have a valid HH:MM format
    to.setTimeFormat();

    auto& table = Table
    << (TRow
        << page1()
        << page2a()
        << page3a()
        << page4()
        )
    << (TRow
        << ""s
        << page2b()
        << page3b()
        );

    // disable all cells
    table.perform( [](Gadget& cell){cell.enable(false);} );

    *this
    << table;

    sc.start(state1);   // starts the state machine
  }


  Gadget& page1() {
    return VBox(".page <big> Device")
    << (Enable() <<= +state1)
    << blink
    << (ButtonBox(".services") << service)
    << (Button(".next")
        << Guard(~service == 0) / (state1 >> state2a)   // done if service set to 0
        << Guard(~service == 1) / (state1 >> state2b)   // done if service set to 1
        );
  }
  
  Gadget& page2a() {
    return VBox(".page <big> Lights")
    << (Enable() <<= +state2a)
    << blink
    << (ListBox(".lights") << light << light_txt)  // selection updates light & light_txt
    << (Button(".next") << (state2a >> state3a));
  }
  
  
  Gadget& page2b() {
    return VBox(".page <big> TV")
    << (Enable() <<= +state2b)
    << blink
    << (ListBox(".channels") << channel << channel_txt)
    << (Button(".next") << (state2b >> state3b));
  }
  
  
  Gadget& page3a() {
    return VBox(".page <big> Time")
    << (Enable() <<= +state3a)
    << blink
    << (Table(".panel")
        << (TRow << "From:"s << from)
        << (TRow << "To:"s  << to)
        )
    << Filler
    << (Button(".next") << (state3a >> state4));
  }
  

  Gadget& page3b() {
    return VBox(".page <big> Time & Language")
    << (Enable() <<= +state3b)
    << blink
    << (HBox(".panel")
        << "Language" << (OptionBox(".languages") << lang << lang_txt)
        )
    << (Table(".panel")
        << (TRow << "From:"s << from)
        << (TRow << "To:"s   << to)
        )
    << Filler
    << (Button(".next") << (state3b >> state4));
  }
  
  
  Gadget& page4() {
    return VBox(".page <big> Running")
    << (Enable() <<= +state4)
    << blink
    << (Table(".final-panel")
        << (TRow << "Playing: "s
            << (""
                << (~service == 0) / ("" + ~light_txt)
                << (~service == 1) / (~channel_txt + " (" + ~lang_txt + ")")
                )
            )
        << (TRow << "From:"s << from)
        << (TRow << "To:"s  << to)
        )
    << Filler
    << (Button(".reset") << (state4 >> state1));
  }
};


GWindow& Examples::makeControlPanel() {
  return *new ControlPanel;
}
