//
//  Charts
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gtrigger.hpp>
#include <gevent.hpp>
#include <gchart.hpp>
#include <gtext.hpp>
#include <gfloat.hpp>
#include <gbind.hpp>
#include <gcontrol.hpp>
#include <gadgetstyle.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(ChartBox)
GIMPLEMENT_GADGET(ChartItem)

GChartBox::GChartBox(GadgetType& t, GString const& spec)
: GChoiceBox(t, spec) {}

GChartBox::GChartBox(GString const& spec)
: GChoiceBox(Type(), spec) {}

GChartBox& GChartBox::clone(GClone const& c) {
  auto* g = new GChartBox;
  //c->choice_ = choice_; done in initClone
  g->initClone(*this, c);
  return *g;
}

void GChartBox::initClone(Gadget& from, GClone const& c) {
  GBox::initClone(from, c);
  if (c & GClone::ShareProps) {
    if (auto* f = from.toChartBox()) {
      choice_ = f->choice_;
      min_ = f->min_;
      max_ = f->max_;
    }
  }
}

Gadget& GChartBox::createDefaultItem(GString const& arg) {
  auto* item = new GChartItem(arg);
  item->setRange(min_, max_);          // !!
  return *item;
}

void GChartBox::add(Gadget* child, int pos) {
  if (!child) {
    error("add","Can't add null child");
    return;
  }
  if (auto* i = child->toValueItem()) i->setRange(min_, max_);
  GChoiceBox::add(child, pos);
}

GChartBox& GChartBox::setRange(float min, float max) {
  min_ = min;
  max_ = max;
  for (auto& it : children()) {
    if (auto* ci = it->toValueItem()) ci->setRange(min_, max_);
  }
  updateLayout();
  return *this;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

GChartItem::GChartItem(GadgetType& t, GString const& spec) : GValueItem(t) {
  // ce constr est necessaire sinon on va appeler le addProp de GValueItem
  value_ = new GFloat;
  addProp(*value_, true);
  *this << on::mousedrag / [this](GEvent&e){mouseDrag(e.asMouseEvent());};

  if (!spec.empty()) initSpec(spec);
}

GChartItem::GChartItem(GString const& spec)
: GChartItem(Type(), spec) {}

GChartItem::GChartItem(float value) : GChartItem(Type(),"") {
  setValue(value);
}

GChartItem& ChartItem(float val) {return *new GChartItem(val);}

GChartItem& GChartItem::clone(GClone const& c) {
  auto* g = new GChartItem;
  g->initClone(*this, c);
  return *g;
}

// NOTE: addProp() different de celui de GValueItem car dans un ChartBox
// le min et le max sont contraints par ceux de la ChartBox
/*
void GChartItem::addProp(GProp& prop, bool add_to_list) {
  auto* p = getParent();

  if (!p || !p->toChartBox()) {
    cerr << "GChartItem::addProp V1 " << &prop<<" " << prop.typeName()<< endl;
    GValueItem::addProp(prop,add_to_list);
  }
  else {
    switch (prop.role()) {

      case GPropType::Number:
        if (!prop.isMutable()) {
          setValue(prop.floatValue());
        }
        else if (auto* num = prop.toNumber()) {
          addProp2(prop, add_to_list);

          if (auto* f = num->toFloat()) {
            cerr << "GChartItem::addProp V2 " << *f <<" " << prop.typeName()<< endl;

            value_ = f;
            num->startNotify(*this, [this](auto&){updateLayout();});
          }
          else if (auto* i = num->toInt()) {
            gsync(*value_, *i);
          }
        }
        update();
        return;

      default:
        GItem::addProp(prop, add_to_list);
        return;
    }
  }
}
*/

}
