//
// Gadget Inspector
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Inspector_hpp
#define Guit_Inspector_hpp
#include <list>
#include <gwindow.hpp>
#include <gtable.hpp>
#include <gscrollbox.hpp>
#include <gchoice.hpp>
#include <gtext.hpp>
#include <gmodes.hpp>
namespace guit {

class GCursor;
class GGlass;

/// Gadget and prop Inspector.
class GInspector : public GWindow {
public:
  ~GInspector();
  GInspector();

  static class GInspector& instance();

  void setInspecting(int mode);
  static int inspecting();

  void inspect(Gadget*);
  static Gadget* inspected();

  GInspector& show(bool show = true) override;
  Gadget* target() {return target_;}

  void setInspectingCB( std::function< void(int) > const& fun);
  void setInspectedCB( std::function< void(Gadget*) > const& fun);

private:
  friend class GBuilder;
  void highlight(Gadget*, bool state);
  void addGadget(GBox& box, Gadget& obj, int ix);
  void deleteGadget(Gadget* obj, bool itself);
  void addProperty(Gadget* parent, GProp* prop);
  void deleteProperty(Gadget* parent, GProp* prop);
  bool moveGadget(GCursor&, Gadget& moved);
  void updateValues();
  void updateChildValues();
  void updatePropValues();
  void moveTarget(Gadget* obj);
  void moveTargetCB(GCursor&, Gadget&);

  GChoice inspectchoice_{0};
  GShow hastarget_, hasparent_, hasundo_;
  GText targettype_, targetext_, targetgeom_;
  GText parenttype_, parentext_;
  GTable propbox_{"#propbox"}, childbox_{"#childbox"};
  GScrollBox propscroll_, childscroll_;
  gprotect<Gadget> target_, targetparent_;
  gptr<GGlass> targetglass_, highlightglass_;
  std::function<void(int)> inspectingcb_{};
  std::function<void(Gadget*)> inspectedcb_{};

  void addUndo(Gadget& obj, GBox* parent);
  void addUndo(GProp& obj, Gadget* parent);
  void undelete();

  struct UndoElem {
    UndoElem(Gadget& obj, GBox* parent, int pos) :
    obj(obj), parent(parent), pos(pos) {}

    UndoElem(GProp& obj, Gadget* parent)
    : obj(obj), parent(parent) {}

    gptr<GObject> obj;
    gptr<Gadget> parent;
    int pos{};
  };

  std::list<UndoElem> undolist_;

};

}
#endif
