//
//  Base class of all classes
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <cstring>  // for memcpy
#include <gapp.hpp>
#include <gadget.hpp>
#include <gcontrol.hpp>
using namespace std;
namespace guit {

#define FINGERPRINT "g/U+i*T?"

void* GObject::operator new(std::size_t sz) {
  void* p = ::operator new(sz);
  //cerr << "new: " << p << " " << sz << " "  << endl;
  ::memcpy(((char*)p) + sizeof(p), FINGERPRINT, sizeof(FINGERPRINT)-1);
  return p;
}

void GObject::operator delete(void* ptr) {
  gerror("GObject::delete","the delete operator does nothing and should not be used",
         (GObject*)ptr);
}

GObject::GObject() {
  bool isplain = true;
  char* p = (char*)this;
  //cerr << "** content: ";
  //for (char* p2 = p; p2 < p+16; ++p2) cerr << *p2;
  //cerr<<endl;
  if (::memcmp(p + sizeof(p), FINGERPRINT, sizeof(FINGERPRINT)-1) == 0) {
    //cerr << "-- created by new: " << (void*)p << endl;
    isplain = false;
  }
  //else //cerr << "++ plain object: " << (void*)p << endl;

  refcount_ = 0;
  omodes_ = {};
  omodes_.plain = isplain;

  if (GCore::plainMode()) {
    omodes_.plain = 1;
    omodes_.immutable = 1;
  }
}

//GObject::~GObject() {
  //gerror("~GObject","deleting an object that is still pointed by another object",this);
//}

void GObject::error(GString const& funname, GString const& msg) const {
  gerror(typeName()+"::"+funname, msg, this);
}

void gaddPtr(GObject* p, bool autodel) {
  if (p && !p->omodes_.plain) {
    ++(p->refcount_);
    if (autodel) p->omodes_.autodel = 1;
  }
}

void gremovePtr(GObject* p) {
  if (p && !p->omodes_.plain) {
    if (p->refcount_ == 0) {
      //throw GFatalError("GObject::remove",
      //                  "attempt to delete an object that was already deleted");

      gerror("GObject::remove",
             "attempt to delete an object that was already deleted", p);
    }
    else if (--(p->refcount_) == 0 && p->omodes_.autodel) {
      ::delete p;
    }
  }
}

}
