//
// Animations
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Anim_hpp
#define Guit_Anim_hpp
#include <gtimer.hpp>
namespace guit {

/// Animation Step
class GAnimStep {
public:
  GAnimStep() = default;
  GAnimStep& operator<<(GString const& str) ;
  GAnimStep& operator<<(GProp& p) {list_.push_back(p); return *this;}
  GAnimStep& operator<<(GProp* p) {list_.push_back(p); return *this;}
  bool empty() const {return list_.empty();}
  std::vector<gptr<GProp>> list_;
};

inline GAnimStep& AnimStep() {return *new GAnimStep();}


/// Animation.
/// Blends prop values when applicable, change then in sequence otherwise.
class GAnim : public GProp {
public:
  enum {Infinite = -1, OneShot = -2};

  ~GAnim();
  GAnim(GTime duration = 0, int repetitions = Infinite);
  GAnim(GString const& spec);

  GAnim* clone(bool) const override {return nullptr;}  ///< can't be cloned.
  GAnim* toAnim() override {return this;}

  /// starts / stops.
  void start(bool yes = true);
  void stop() {start(false);}

  /// restarts from the current step.
  virtual void restart();

  GAnim& operator<<(GAnimStep& prop);
  GAnim& operator<<(GString const& proplist);
  GAnim& operator<<(GProp& prop);
  GAnim& operator<<(GProp* prop);
  GAnim& operator<<(GFunction const& fun);

  int    repetitions() const {return count_;}
  GAnim& setRepetitions(int count);

  bool  isRunning() const;
  GTime duration() const {return duration_;}
  GTimer* timer() {return timer_;}
  GString stringValue() const override;

  using GType = GPropType_<GAnim,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  bool isEquivalent(GProp const& other) const override {return false;}
  void apply(Gadget*, GRender&, Specif) override;
  void addPropsTo(Gadget&);
  void removePropsFrom(Gadget&);

protected:
  friend class GVarProp;
  virtual void init();
  void timeoutCB();
  void fireCallbacks(GEvent&);
  GTime duration_{}, elapsed_{}, timeout_{};
  int count_{1};
  gptr<GTimer> timer_;
  GPropList calls_;
  GAnimStep props_;
  std::vector<GAnimStep*> steps_;
  void updateProps(GAnimStep& from, GAnimStep& to, float alpha);
};

/// Returns a GAnim created in the heap.
inline GAnim& Anim(GTime duration = 0, int count = GAnim::Infinite) {
  return *new GAnim(duration, count);
}

}
#endif
