//
//  Float Prop
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef GuitFloat_hpp
#define GuitFloat_hpp
#include <gnumber.hpp>
namespace guit {

/// Float prop.
class GFloat : public GNumber_<float> {
public:
  using Super = GNumber_<float> ;

  GFloat();
  GFloat(float value);
  GFloat(float value, float min, float max = highest());
  GFloat(GFloat const& value);
  GFloat(GString const& value);  ///< null if _value_ is not a valid floting number.
  GFloat* clone(bool copy_value) const override;

  GFloat* toFloat() override {return this;}

  /// Future value in an Active Expression (see GExpr).
  auto operator~() {return GNumPropRef_<GFloat>(*this);}

  /// Binds this prop to an Active Expression (see GExpr).
  GFloat& operator<<=(GNumExpr&);

  GFloat& setRange(float min, float max) {Super::setRange(min,max); return *this;}
  GFloat& setRangeF(float min, float max) override {Super::setRange(min,max); return *this;}

  GFloat& operator=(float value) {set(value,false); return *this;}
  GFloat& operator=(GFloat const& value) {set(value,false); return *this;}
  GFloat& operator=(GString const& value) {set(value); return *this;}

  GFloat& operator++()    {increase(); return *this;}
  GFloat& operator++(int) {increase(); return *this;}
  GFloat& operator--()    {increase(); return *this;}
  GFloat& operator--(int) {increase(); return *this;}
  GFloat& operator+=(float delta) {increase(delta); return *this;}
  GFloat& operator-=(float delta) {increase(-delta); return *this;}

  using Super::increase;
  void increase() override {increase(+1.f);}
  void decrease() override {increase(-1.f);}

  using GType = GPropType_<GFloat,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}
};

/// Return a GFloat created in the heap.
/// @{
GFloat& Float();
inline GFloat& Float(float value) {return *new GFloat(value);}
GFloat& Float(float value, float min, float max = GFloat::highest());
GFloat& Float(GString const& value);
/// @}

}
#endif
