//
// Textual format
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef GuitFormat_hpp
#define GuitFormat_hpp
#include <gprop.hpp>
namespace guit {

/// Text Format
class GFormatValue {
public:
  GFormatValue();
  bool operator==(GFormatValue const&) const;
  bool operator!=(GFormatValue const&) const;
  int8_t precision{}, nbchars{};
  char fillchar{};
  /*
   struct {
   int8_t underline:1, overline:1, linethrough:1;
   } decoration;
   float lineheight;
   */
};

bool gconvert(GFormatValue const& from, GFormatValue& to);
bool gconvert(GString const& from, GFormatValue& to);
bool gconvert(GFormatValue const& from, GString& to);


/// Format Prop.
class GFormat : public GVarProp_<GFormatValue> {
public:
  using Super = GVarProp_<GFormatValue>;

  /// format in the form: "precision nbchars fillchar".
  GFormat(GString const& format = "");
  GFormat(GFormatValue const&);
  GFormat(GFormat const&);
  GFormat* clone(bool copy_value) const override;

  using GType = GPropType_<GFormat,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  void apply(Gadget*, GRender&, Specif) override;
  void updateGadget(Gadget&) override;
};

}
#endif
