//
// Icon of a gadget
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gicon.hpp>
#include <ggraphics.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_PROP_TYPE(Icon, GPropType::Icon)

static GCore::BeginPlainObjects begin;
GIcon
GIcon::check(CheckSymbol, GColor::guitblue, GColor::none),
GIcon::radio(RadioSymbol, GColor::guitblue, GColor::none),
GIcon::combo(ComboSymbol, GColor::guitblue, GColor::white),
GIcon::up(UpSymbol, GColor::guitblue, GColor::white),
GIcon::down(DownSymbol, GColor::guitblue, GColor::white),
GIcon::left(LeftSymbol, GColor::guitblue, GColor::white),
GIcon::right(RightSymbol, GColor::guitblue, GColor::white),
GIcon::updown(UpDownSymbol, GColor::guitblue, GColor::orange);
static GCore::EndPlainObjects end;


void GIcon::Image(GRenderGraphics& g, GRender& r, GRect const& iframe) {}

void GIcon::XPM(GRenderGraphics& g, GRender& r, GRect const& iframe) {}

void GIcon::CheckSymbol(GRenderGraphics& g, GRender& r, GRect const& iframe) {
  auto& obj = r.gadget();

  g.setThickness(1.);
  if (obj.isChecked()) g.setColor(*r.icon_->fg);
  else g.setColor(GColor::dimgray);

  g.drawRoundRect(GRect(iframe.x, iframe.y, iframe.w-3, iframe.h-3), 1., 1.);

  if (obj.isChecked()) {
    g.drawRoundRect(GRect(iframe.x+1, iframe.y+1, iframe.w-5, iframe.h-5), 1., 1.);
  }
}


void GIcon::RadioSymbol(GRenderGraphics& g, GRender& r, GRect const& iframe) {
  auto& obj = r.gadget();

  g.setThickness(1.);
  if (obj.isChecked()) g.setColor(*r.icon_->fg);
  else g.setColor(GColor::dimgray);

  g.drawOval(GRect(iframe.x, iframe.y, iframe.w-3, iframe.h-3));

  if (obj.isChecked()) {
    g.drawOval(GRect(iframe.x+1, iframe.y+1, iframe.w-5, iframe.h-5));
  }
}

 void GIcon::drawHArrow(GRenderGraphics& g, GRect const& frame,
                       float lx, float rx, float ty, float by) {
  g.drawLine(frame.x + frame.w * lx, frame.y + frame.h * ty,
             frame.x + frame.w * rx, frame.y + frame.h * 0.5f);

  g.drawLine(frame.x + frame.w * rx, frame.y + frame.h * 0.5f,
             frame.x + frame.w * lx, frame.y + frame.h * ty);
}

 void GIcon::drawVArrow(GRenderGraphics& g, GRect const& frame,
                       float lx, float rx, float ty, float by) {
  g.drawLine(frame.x + frame.w * lx, frame.y + frame.h * ty,
             frame.x + frame.w * 0.5f, frame.y + frame.h * by);

  g.drawLine(frame.x + frame.w * 0.5f, frame.y + frame.h * by,
             frame.x + frame.w * rx, frame.y + frame.h * ty);
}

void GIcon::LeftSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  g.setThickness(1.);
  g.setColor(*r.icon_->fg);
  if (r.obj_->isActive()) g.setBgColor(*r.icon_->fg);
  g.drawRoundRect(GRect(frame.x, frame.y, frame.w, frame.h), 1., 1.);

  g.setThickness(2.);
  if (r.obj_->isActive()) g.setColor(*r.icon_->bg);
  drawHArrow(g, frame, 0.25f, 0.75f, 0.7f, 0.3f);
}


void GIcon::RightSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  g.setThickness(1.);
  g.setColor(*r.icon_->fg);
  if (r.obj_->isActive()) g.setBgColor(*r.icon_->fg);
  g.drawRoundRect(GRect(frame.x, frame.y, frame.w, frame.h), 1., 1.);

  g.setThickness(2.);
  if (r.obj_->isActive()) g.setColor(*r.icon_->bg);
  drawHArrow(g, frame, 0.25f, 0.75f, 0.3f, 0.7f);
}


void GIcon::UpSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  g.setThickness(1.);
  g.setColor(*r.icon_->fg);
  if (r.obj_->isActive()) g.setBgColor(*r.icon_->fg);
  g.drawRoundRect(GRect(frame.x, frame.y, frame.w, frame.h), 1., 1.);

  g.setThickness(2.);
  if (r.obj_->isActive()) g.setColor(*r.icon_->bg);
  drawVArrow(g, frame, 0.25f, 0.75f, 0.7f, 0.3f);
}


void GIcon::DownSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  g.setThickness(1.);
  g.setColor(*r.icon_->fg);
  if (r.obj_->isActive()) g.setBgColor(*r.icon_->fg);
  g.drawRoundRect(GRect(frame.x, frame.y, frame.w, frame.h), 1., 1.);

  g.setThickness(2.);
  if (r.obj_->isActive()) g.setColor(*r.icon_->bg);
  drawVArrow(g, frame, 0.25f, 0.75f, 0.3f, 0.7f);
}

void GIcon::ComboSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  if (r.obj_->isChecked()) UpSymbol(g, r,frame);
  else DownSymbol(g, r,frame);
}

void GIcon::UpDownSymbol(GRenderGraphics& g, GRender& r, GRect const& frame) {
  // draw background to hide numbers if too large
  g.setThickness(1.);
  //g.setColor(GColor::white);
  //g.drawRect(frame);

  // draw arrows
  g.unsetBgColor();
  g.setColor(*r.icon_->fg);
  g.setThickness(2.);
  GIcon::drawVArrow(g, frame, 0.2f, 0.8f, 0.4f, 0.05f); //up
  GIcon::drawVArrow(g, frame, 0.2f, 0.8f, 0.6f, 0.95f); // down

  if (r.obj_->isActive()) {
    g.setColor(*r.icon_->bg);
    if (r.icon_->area == 1)
      GIcon::drawVArrow(g, frame, 0.2f, 0.8f, 0.4f, 0.05f); //up
    else if (r.icon_->area == 2)
      GIcon::drawVArrow(g, frame, 0.2f, 0.8f, 0.6f, 0.95f); // down
  }
}


GIconValue::~GIconValue() {
  if (texture) {
    //cerr << "GIconValue::~GIconValue() "<< endl;
    GGraphics::deleteTexture(texture);
  }
}

bool GIconValue::operator==(const GIconValue& other) const {
  return drawfun == other.drawfun && filename == other.filename && xpm == other.xpm;
}

bool GIconValue::operator!=(const GIconValue& other) const {
  return drawfun != other.drawfun || filename != other.filename || xpm != other.xpm;
}

bool gconvert(GString const& from, GIconValue& to) {
  to.filename.clear();
  to.xpm = nullptr;
  to.texture = 0;
  to.fg = {};
  to.bg = {};

  GString token;
  size_t pos{};
  if (strtoken(from, token, pos, "") < 0) return false;

  if (token == "check") {
    to = GIcon::check();
  }
  else if (token == "radio") {
    to = GIcon::radio();
  }
  else if (token == "combo") {
    to = GIcon::combo();
  }
  else if (token == "up") {
    to = GIcon::up();
  }
  else if (token == "down") {
    to = GIcon::down();
  }
  else {
    to.drawfun = GIcon::Image;
    to.status = GIconValue::Unloaded;
    to.filename = from;
  }

  if (strtoken(from, token, pos, "") < 0) return true;
  to.fg = new GColor(token);

  if (strtoken(from, token, pos, "") < 0) return true;
  to.bg = new GBackground(token);

  return true;
}

bool gconvert(GIconValue const& from, GIconValue& to) {
  to = from;        // note: partage status et texture !
  return true;
}

bool gconvert(GIconValue const& from, GString& to) {
  to = from.xpm ? "Pixmap" : from.filename;     // INCOMPLET !!!!
  return true;
}

// - - - -

GIcon::GIcon() {}

GIcon::GIcon(GString const& filename) {
  set(filename);
}

GIcon::GIcon(const char** xpm) {
  pmodes_.isSet = 1;
  value_.drawfun = GIcon::XPM;
  value_.xpm = xpm;
}

// for predef icons
GIcon::GIcon(DrawFun fun, GColor& fg, GColor& bg) {
  pmodes_.isSet = 1;
  value_.status = GIconValue::Loaded;
  value_.drawfun = fun;
  value_.fg = &fg;
  value_.bg = &bg;
}

GIcon::GIcon(GIconValue const& value) : Super(value) {}

GIcon* GIcon::clone(bool copy_value) const {
  return copy_value ? new GIcon(value_) : new GIcon;
}

void GIcon::updateGadget(Gadget& g) {
  g.updateLayout();
}

void GIcon::apply(Gadget*, GRender& r, Specif specif) {
  if (specif < r.specif[GPropType::Icon]) return;
  r.specif[GPropType::Icon] = specif;
  
  r.icon_ = &value_;
}
  
}
