//
// Margins of a gadget
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <sstream>
#include <gevent.hpp>
#include <gmargin.hpp>
#include <gpropspec.hpp>
#include <ggraphics.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
#include <gnative.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_PROP_TYPE(Margin, GPropType::Margin)

GMarginValue::GMarginValue()
: left(0), right(0), top(0), bottom(0) {}

GMarginValue::GMarginValue(float left, float right, float top, float bottom)
: left(left), right(right), top(top), bottom(bottom) {}

bool GMarginValue::operator==(const GMarginValue& other) const {
  return
  left == other.left && right == other.right
  && top == other.top && bottom == other.bottom;
}

bool GMarginValue::operator!=(const GMarginValue& other) const {
  return
  left != other.left || right != other.right
  || top != other.top || bottom != other.bottom;
}

static GPropSpec marginspec_{
  new GNumFieldSpec<GMarginValue,float>(&GMarginValue::left),
  new GNumFieldSpec<GMarginValue,float>(&GMarginValue::right),
  new GNumFieldSpec<GMarginValue,float>(&GMarginValue::top),
  new GNumFieldSpec<GMarginValue,float>(&GMarginValue::bottom)
};

bool gconvert(GString const& from, GMarginValue& to) {
  to = {};
  //to.flex1 = to.flex2 = -1;
  //to.mode = GFlex::Undef;
  int n = marginspec_.toValue(from, &to);
  switch (n) {
    case 1:
      to.top = to.bottom = to.right = to.left;
      break;
    case 2:
      to.top = to.bottom = to.right; to.right = to.left;
      break;
  }

  if (n <= 0) gerror("Margin","invalid keyword or value: " + from);
  return n > 0;
}

bool gconvert(GMarginValue const& from, GString& to) {
  to = marginspec_.toString(&from);
  return true;
}

// - - - - -

GMargin::GMargin() = default;

GMargin::GMargin(float leftright, float topbottom)
: Super(GMarginValue(leftright, leftright, topbottom, topbottom)) {}

GMargin::GMargin(float left, float right, float top, float bottom)
: GVarProp_<GMarginValue>(GMarginValue(left, right, top, bottom)) {}

GMargin:: GMargin(GMarginValue const& value)
: GVarProp_<GMarginValue>(value) {}

GMargin::GMargin(GString const& format) {
  set(format);
}

GMargin* GMargin::clone(bool copy_value) const {
  return copy_value ? new GMargin(value_) : new GMargin;
}

void GMargin::updateGadget(Gadget& g) {
  g.updateLayout();
}

void GMargin::apply(Gadget*, GRender& r, Specif specif) {
  if (specif < r.specif[GPropType::Margin]) return;
  r.specif[GPropType::Margin] = specif;

  r.margin_ = &value_;
}

}
