//
//  Guit GUI Toolkit Examples.
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <gshape.hpp>
using namespace std;
using namespace guit;

/// Files (CSSs, icons, etc.) that do not start with /  \  . and magic characters ~ : ! are searched in
/// the application resource directory (MacOSX only), then in guit-dir/exec-name where guit-dir is
/// the Guit directory of the user, and exec-name the name of the executable. guit-dir is located here:
/// - MacOS: ~/Library/Guit
/// - Linux: ~/.guit
/// - Windows: ~/Guit
///
/// Magic characters must be the first character of a file name:
/// - ~  is the home directory of the user
/// - !  is the Guit directory of the user (ie guit-dir)
/// - :  is the first resource directory (ie guit-dir/exec-name, except for MacOS applications where
/// it's the application resource directory, ie application/Content/Resources)


class Tutorial : public GWindow {
public:

  Tutorial() : GWindow("#tutorial Tutorial") {

    *this
    << QuitDialog()
    << (MenuBar()
        << ("File" << filemenu())
        )
    << (ToolBar()
        << (".quit"     << []{ GApp::close(); })
        << ("Inspector" << []{ GApp::showInspector(); })

        )
    << (TabBox()
        << tutorial1()
        << tutorial2()
        << tutorial3()
        << tutorial4()
        << tutorial5()
        << tutorial6()
        << tutorial7()
        << tutorial8()
        << tutorial9()
        );
  }

private:

  GBox& tutorial1()
  {
    auto& box1 = Box("Box")    // don't forget & symbol (reference)
    << "Apples"       // as: << Label("Apples")
    << "Oranges"
    << "Mangoes";

    auto& box2 = ButtonBox("ButtonBox")
    << "Apples"       // as: << Button("Apples")
    << "Oranges"
    << "Mangoes";

    auto& box3 = ListBox("ListBox")
    << "Apples"       // as: << Item("Apples")
    << "Oranges"
    << "Mangoes";

    auto& box4 = ComboBox("ComboBox")
    << "Apples"
    << "Oranges"
    << "Mangoes";

    auto& box5 = ListBox("ListBox")
    << "{background: yellow} Apples"
    << "{background: orange} Oranges"
    << "{background: red} Mangoes";

    auto& box6 = ChartBox("Chart")
    << "{background: yellow; value: 30} Apples"   // as: << ChartItem("...")
    << "{background: orange; value: 60} Oranges"
    << "{background: red; value: 40} Mangoes";

    return HBox(".row Intro")
    << box1 << box2 << box3 << box4 << box5 << box6;
  }

  GMenu& filemenu() {
  return Menu()
    << (".open" << [] { cout << "Open" << endl; })
    << (".save" << [] { cout << "Save" << endl; })
    << Separator
    << (".quit" << [] { GApp::close(); });
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial2()
  {
    GStrings fruits {
      "Apples", "Oranges", "Mangoes"
    };

    GStrings ch_fruits {
      "{background: yellow; value: 30} Apples",
      "{background: orange; value: 60} Oranges",
      "{background: red; value: 40} Mangoes"
    };

    auto& choice = Choice();    // don't forget & sign (reference)

    auto& row1 = HBox(".row")
    << (Box("Box")             << fruits)
    << (ButtonBox("ButtonBox") << fruits << choice)
    << (RadioBox("RadioBox")   << fruits << choice)
    << (ChoiceBox("ChoiceBox") << fruits << choice)
    << (ListBox("ListBox")     << fruits << choice)
    << (OptionBox("OptionBox") << fruits << choice)
    << (ComboBox("ComboBox")   << fruits << choice);

    auto& row2 = HBox(".row")
    << (ChartBox("{flow:row} Row")       << ch_fruits << choice)
    << (ChartBox("{flow:column} Column") << ch_fruits << choice)
    << (ChartBox("{flow:pie} Pie")       << ch_fruits << choice)
    << (TabBox("{size:150 150} TabBox")  << fruits << choice);

    return VBox("Boxes") << row1 << row2;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial3()
  {
    auto& choice = Choice();

    auto& row1 = HBox(".row")
    << (Box(".fruits Box"))
    << (ButtonBox(".fruits ButtonBox"))
    << (RadioBox(".fruits RadioBox")   << choice)
    << (ChoiceBox(".fruits ChoiceBox") << choice)
    << (ListBox(".fruits ListBox")     << choice)
    << (OptionBox(".fruits OptionBox") << choice)
    << (ComboBox(".fruits ComboBox")   << choice);

    auto& row2 = HBox(".row")
    << (ChartBox(".ch-fruits {flow:row} Row")       << choice)
    << (ChartBox(".ch-fruits {flow:column} Column") << choice)
    << (ChartBox(".ch-fruits {flow:pie} Pie")       << choice)
    << (TabBox(".fruits {size:150 150} TabBox")   << choice);

    return VBox("CSS") << row1 << row2;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial4()
  {
    auto& index  = Int();
    auto& text   = Text();
    auto& chosen = Text();

    auto& choice = Choice(1, 0, 2);
    choice << [&]{ index = choice; };              // callback on choice prop

    auto& chart1 = ChartBox("{flow:row}")
    << choice
    << (".apples"  << [&]{ text = "Apples"; })     // callback on item child
    << (".oranges" << [&]{ text = "Oranges"; })
    << (".mangoes" << [&]{ text = "Mangoes"; });

    auto& chart2 = ChartBox("{flow:column}")
    << choice
    << ".apples"
    << ".oranges"
    << ".mangoes";
    chart2 << [&]{ chosen = chart2.chosenText(); };  // callback on chart box

    auto& chart3 = ChartBox("{flow:pie}")
    << choice
    << ".apples"
    << ".oranges"
    << ".mangoes";

    auto& selection = Table("{size:15c} Selection")
    << (TRow() << "Index:"  << index)
    << (TRow() << "Text:"   << text)
    << (TRow() << "Chosen:" << chosen);

    return HBox(".row Callbacks")
    << chart1 << chart2 << chart3 << selection;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial5()
  {
    auto& choice = Choice(1, -1, 2);
    auto& bg1  = Background();
    auto& bg2  = Background();
    auto& bg3  = Background();
    auto& val1 = Float();
    auto& val2 = Float();
    auto& val3 = Float();

    auto& chart1 = ChartBox("{flow:row}")
    << choice
    << (".apples"  << bg1 << val1)
    << (".oranges" << bg2 << val2)
    << (".mangoes" << bg3 << val3);

    auto& chart2 = ChartBox("{flow:column}")
    << choice
    << (".apples"  << bg1 << val1)
    << (".oranges" << bg2 << val2)
    << (".mangoes" << bg3 << val3);

    auto& chart3 = ChartBox("{flow:pie}")
    << choice
    << (".apples"  << bg1 << val1)
    << (".oranges" << bg2 << val2)
    << (".mangoes" << bg3 << val3);

    auto& values = Table(".values Values")
    << (TRow() << "Apples"  << val1 << (Slider() << val1))
    << (TRow() << "Oranges" << val2 << (Slider() << val2))
    << (TRow() << "Mangoes" << val3 << (Slider() << val3))
    << (TRow() << "Chosen"
        << (SpinItem() << choice)
        << (OptionBox(".fruits") << choice)
        );

    return HBox(".row Syncing")
    << chart1 << chart2 << chart3 << values;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial6()
  {
    auto& euros   = Float(100);
    auto& dollars = Float();
    auto& rate    = Float(1.12f);

    dollars <<= ~euros * ~rate;     // active expressions that are auto-updated
    euros   <<= ~dollars / ~rate;   // when the variables preceeded by a ~ change

    auto& conv = Table(".darkpanel Euros to Dollars")
    << (TRow() << "<b>Euros:"   << euros)
    << (TRow() << "<b>Rate:"    << rate)
    << (TRow() << "<b>Dollars:" << dollars);


    auto &a = Float(), &b = Float(), &c = Float(), &d = Float();

    auto& calc = Table(".darkpanel .calculations Calculations")
    << (TRow()
        << a << "+" << b << " = " << (~a + ~b)
        )
    << (TRow()
        << c << "x" << d << " = "  << (~c * ~d)
        );


    auto& role  = Text("Columbo");
    auto& actor = Text("Peter Falk");
    auto& rcol  = Text("red");
    auto& acol  = Text("blue");

    auto& movies = Table(".darkpanel Multi-Text")
    << (TRow() << "<b>Role:"  << role  << rcol)
    << (TRow() << "<b>Actor:" << actor << acol)
    << (Label("{margin: 0 6}")
        << ("The role of <c:" + ~rcol +"><b>" + ~role + "</b></c>"
            " is played by <c:" + ~acol + "><b>" + ~actor)
        );

    return HBox(".row Expressions")
    << (HBox(".row {align: center stretch}")
        << conv << calc << movies
        );
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial7()
  {
    auto& speed = Int(0, 0, 200);

    auto& speedbox = VBox(".darkpanel {size:150} Speed")
    << (HBox()
        << "0" << (Slider(".lightslider") << speed) << "200"
        )
    << ""
    << (Label("{font:big; align:c c}")
        << (~speed == 0)  / "Stopped"
        << (~speed > 0  && ~speed <= 90)  / ("<c=green>Safe: " + ~speed +" km/h")
        << (~speed > 90 && ~speed <= 130) / ("<c=blue>Fast: "+ ~speed +" km/h")
        << (~speed > 130) / ("<c=red>Danger! "+ ~speed +" km/h")
        );


    auto& kind = Choice();
    auto& card = Int();
    auto& pass = Text();

    auto& entrance = VBox(".darkpanel {size:180} Entrance")
    << (HBox()
        << "Status:" << (RadioBox() << "Child" << "Adult" << "Member" << kind)
        )
    << (~kind == 0) / "<b>Free"
    << (~kind == 1) / (HBox() << "<b>Credit Card:" << card)
    << (~kind == 2) / (HBox() << "<b>Password:"    << pass);


    auto& fname = Text();
    auto& lname = Text();
    auto& age   = Int();
    auto& done  = Bool();

    auto& info = Table()
    << (TRow() << "First Name:" << fname)
    << (TRow() << "Last Name:"  << lname)
    << (TRow() << "Age:"        << age)
    << (TRow()
        << ""
        << (Button("Suscribe")
            << Guard(~fname == "") / [&]{ fname.blink("Required"); }
            << Guard(~lname == "") / [&]{ lname.blink("Required"); }
            << Guard(~age <= 0)    / [&]{ age.blink("Required"); }
            << Otherwise()         / [&]{ done = true;}
            )
        );

    auto& suscribe = VBox(".darkpanel {size:200} Suscribe")
    << (~done == false) / info
    << (~done == true)  / (VBox(".welcome") << "Welcome " << (~fname + " " +~lname));

    return HBox(".row Conditions")
    << speedbox << entrance << suscribe;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial8()
  {
    auto& seconds = Int(0, 0, 100);
    auto& ctimer = Timer(1000);  // fired every second, indefinitely

    ctimer
    << on::change / [&]{ seconds--; }
    << on::done   / [&]{ blink(); dring_anim(); };

    auto& cooking_timer = VBox("#cooking-timer")
    << (SpinItem() << seconds)
    << (Button("Start") << [&]{ ctimer.setRepetitions(seconds); ctimer.start(); });


    auto& ltimer = Timer(2000);
    // beat increased each time the timer is fired, reset to 0 when it reaches 3
    auto& beat = ltimer.addBeat(1, 3);

    auto& traffic_lights = VBox("#traffic-lights")
    << (OvalShape() << (~beat == 0) / Background("green"))
    << (OvalShape() << (~beat == 1) / Background("orange"))
    << (OvalShape() << (~beat == 2) / Background("red"))
    << (Label()
        << (~beat == 0) / "<color:green>Go"
        << (~beat == 1) / "<color:orange>Wait"
        << (~beat == 2) / "<color:red>Stop"
        )
    << (Button("Cheat") << [&]{beat++;});

    return HBox(".row {gap:100; align: c c} Timers+Anims")
    << cooking_timer << traffic_lights
    << on::show / [&]{ ltimer.start(); }
    << on::hide / [&]{ ltimer.stop(); };
  }

  void dring_anim() {
    static gptr<GAnim>  anim;
    static gptr<GLabel> anim_label;

    if (!anim) {
      anim = Anim(3000, 1)   // 3sec, once
      << (AnimStep() << Pos(200,200) << Color("red"))
      << (AnimStep() << Pos(300,300) << Color("orange"))
      << (AnimStep() << Pos(250,250) << Color("red"))
      << (AnimStep() << Pos(300,200) << Color("orange"))
      << (AnimStep() << Pos(200,200) << Color("none"));

      *this << (anim_label = Label("#dring DRING!") << anim);

      // hide label once anim done
      anim << on::done / []{ anim_label->show(false); };
    }

    anim->start();
    anim_label->show();
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  GBox& tutorial9()
  {
    auto& text = Text("Blah blah blah");
    auto& font = Font("14 arial, sans-serif");

    auto& text_box = VBox("{size: 300 a}")
    << (ScrollBox()
        << (TextArea << text << font)
        )
    << (HBox("{flex: 1 0}")
        << "<b>Font:"  << (Text() ^= font)   // syncs this Text with font
        );

    // the images must be stored:
    // - MacOS: in the application Content directory or ~/Library/tutorial
    // - Linux: in ~/guit/tutorial
    // - Windows: in ~/Guit/tutorial

    auto& chosen = Text();

    auto& page = VBox("{size: 300 a; background: white}")
    << ".header Click on your preferred fruit:"
    << (ListBox()
        // this item contains an image
        << "{tip:oranges} Oranges <img src=oranges.jpg height=30>"
        << "{tip:mangoes} Mangos <img src=mangoes.jpg height=30>"
        // the tip contains an image
        << "{tip:apples <img src=apples.jpg height=30>} Apples <img src=apples.jpg height=30>"
        << "{tip:bananas} Bananas <img src=bananas.jpg height=30>"
        << "{tip:peaches} Peaches <img src=peaches.jpg height=30>"
        // called when one item is clicked
        << [&](GEvent& e) { if (e.chosen()) chosen = e.chosen()->text(); }
    )
    << (".header" << ("Chosen fruit: <color:red>" + ~chosen));

    return HBox(".row {gap:50; align: center stretch} Text+Images")
    << text_box << page;
  }

};

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

int main(int argc, const char *argv[]) {
  GApp app(argc, argv);
  app << "tutorial.css";  // loads CSS file

  app.inspector();  // allows inspecting by Meta or Cmd-clicking on a gadget

  return app.start(new Tutorial);  // starts the event loop (must be last)
}
